#include "simpleOptions.h"
#include <math.h>
#include <itkImageRegistrationMethod.h>
#include <itkTranslationTransform.h>
#include <itkMattesMutualInformationImageToImageMetric.h>
#include <itkLinearInterpolateImageFunction.h>
#include <itkRegularStepGradientDescentOptimizer.h>
#include <itkImage.h>

#include <itkImageFileReader.h>
#include <itkImageFileWriter.h>
#include <itkExtractImageFilter.h>

#include <itkDiscreteGaussianImageFilter.h>
#include <itkResampleImageFilter.h>
#include <itkCastImageFilter.h>
#include <itkNormalizeImageFilter.h>
#include <itkCommand.h>
#include <itkScaleTransform.h>
#include <itkGradientMagnitudeRecursiveGaussianImageFilter.h>
#include <itkMinimumMaximumImageCalculator.h>

class MattesMutualInformationRegistrationApplication : public itk::Object
{
private:
	unsigned int debugflag;
	unsigned int writeoutinitialflag;
	unsigned int spatialSamples;		//Metric
	unsigned int bins;					//Metric
	unsigned int numberOfIterations;	//Optimizer
	double maxStep;						//Optimizer
	double minStep;						//Optimizer
	double sigmaFixed;
	double sigmaMoving;
	double initX;
	double initY;
	double initZ;
	unsigned int operation;
	char *movingFilename;
	char *fixedFilename;
	char *outputFilename;

protected:

public:
	MattesMutualInformationRegistrationApplication()
	{
		//Default-values
		debugflag = 0;
		writeoutinitialflag = 0;
		spatialSamples = 8000;		//Metric
		bins = 128;					//Metric
		numberOfIterations = 50;	//Optimizer
		maxStep = 5;				//Optimizer
		minStep = 0.1;				//Optimizer
		sigmaFixed = 3.0;
		sigmaMoving = 3.0;
		initX = 0.0;
		initY = 0.0;
		initZ = 0.0;
		movingFilename = "";
		fixedFilename = "";
		outputFilename = "";
		operation = 0; //Default no blurring or gradient
	};
	virtual ~MattesMutualInformationRegistrationApplication() {};
	typedef MattesMutualInformationRegistrationApplication	Self;
	typedef itk::Object								Superclass;
	typedef itk::SmartPointer<Self>					Pointer;
	typedef itk::SmartPointer<const Self>				ConstPointer;

	static const unsigned int	Dimension = 3;
	typedef unsigned short		InputPixelType;
	typedef unsigned short		OutputPixelType;
	typedef double				PixelType;

	typedef itk::Image< InputPixelType, Dimension >		FixedImageType;
	typedef itk::Image< InputPixelType, Dimension >		MovingImageType;
	typedef itk::Image< PixelType, Dimension >			ImageType;
	typedef itk::Image< OutputPixelType, Dimension >	OutputImageType;

	typedef itk::TranslationTransform< double, Dimension > TransformType;
	typedef itk::RegularStepGradientDescentOptimizer OptimizerType;
	typedef itk::MattesMutualInformationImageToImageMetric< ImageType, ImageType > MetricType;
	typedef itk::LinearInterpolateImageFunction< ImageType, double > InterpolatorType;
	typedef itk::ImageRegistrationMethod< ImageType, ImageType > RegistrationType;
		typedef RegistrationType::ParametersType ParametersType;

	typedef itk::ImageFileReader< FixedImageType >	FixedImageReaderType;
	typedef itk::ImageFileReader< MovingImageType > MovingImageReaderType;

	typedef itk::ResampleImageFilter< FixedImageType, FixedImageType > ResampleFixedFilterType;
	typedef itk::ResampleImageFilter< MovingImageType, MovingImageType > ResampleMovingFilterType;
	
	typedef itk::CastImageFilter< FixedImageType, ImageType > FixedCastFilterType;
	typedef itk::CastImageFilter< MovingImageType, ImageType > MovingCastFilterType;

	typedef itk::DiscreteGaussianImageFilter< ImageType, ImageType > GaussianFilterType;
	typedef itk::ResampleImageFilter< ImageType, ImageType > InitialImageWriterResampleFilterType;

	typedef itk::ResampleImageFilter< MovingImageType, FixedImageType > ResampleFinalFilterType;

	typedef itk::GradientMagnitudeRecursiveGaussianImageFilter< ImageType, ImageType > GradMagFilterType;

	typedef itk::CastImageFilter< FixedImageType, OutputImageType > CastFilterType;
	typedef itk::ImageFileWriter< OutputImageType >  WriterType;

	int begin();
	void setDebugFlag(unsigned int dflag);
	void setWriteOutInitialFlag(unsigned int wflag);
	void setMovingFilename(char *filename);
	void setFixedFilename(char *filename);
	void setOutputFilename(char *filename);
	char *getMovingFilename();
	char *getFixedFilename();
	char *getOutputFilename();
	void setSpatialSamples(unsigned int samples);
	void setNumberOfBins(unsigned int bins);
	void setNumberOfIterations(unsigned int iterations);
	void setMaxStep(double maxStep);
	void setMinStep(double minStep);
	void setSigmaFixed(double gradMagSmoothSigma);
	void setSigmaMoving(double gradMagSmoothSigma);
	void setInitX(double initX);
	void setInitY(double initY);
	void setInitZ(double initZ);
	void setOperation(unsigned int operation);

	void usage()
	{
		std::cerr << "Usage: RegApp1 -f <fixedImageFile>  -m <movingImageFile> " << std::endl;
	    std::cerr << std::endl;
		std::cerr << "       RegApp1 -f <fixedImageFile>  -m <movingImageFile> " << std::endl;
		std::cerr << "               [ -o <outputImagefile> ] " << std::endl;
		std::cerr << "               [ -i <numberOfIterations> ] " << std::endl;
		std::cerr << "               [ -X <maxOptimizerStep> ] " << std::endl;
		std::cerr << "               [ -n <minOptimizerStep> ] " << std::endl;
	    std::cerr << "               [ -s <numberOfSpatialSamples> ] " << std::endl;
		std::cerr << "               [ -b <numberOfHistogramBins> ] " << std::endl;
		std::cerr << "               [ -v <gradMagSmoothSigmaFixed> ] " << std::endl;
		std::cerr << "               [ -V <gradMagSmoothSigmaMoving> ] " << std::endl;
		std::cerr << "               [ -x <initialOffsetX> ] " << std::endl;
	    std::cerr << "               [ -y <initialOffsetY> ] " << std::endl;
		std::cerr << "               [ -z <initialOffsetZ> ] " << std::endl;
		std::cerr << "               [ -a <registrationOperation> ] " << std::endl;
		std::cerr << "               [ -d <debugFlag] " << std::endl;
		std::cerr << "               [ -w <writeOutInitial]" << std::endl;
	    exit(1);
	}
};