// File:    SimpleOptions.h
// Author:  Brodie Thiesfield
// Licence: Freeware (see below)
//
// FEATURES
//  * accepts getopt command line specification
//  * simple
//  * platform independent
//  * case dependent or independent matching
//
// LIMITATIONS
//  * it isn't getopt
//  * all options on the command line must use flags
//      ie  "program [-o] <input_file>" won't work
//      use "program [-o] -i <input_file>" instead
//
// NOTES
//  * uses exceptions
//  * if using case-insensitive option matching, ensure that the string
//    passed through as a_pszOptions is all lowercase
//
// EXAMPLE USAGE
//
//    try
//    {
//        CSimpleOptions cl( argc, argv, "Oa:" );
//        while ( cl.next() )
//        {
//            switch ( cl.option() )
//            {
//            case 'O':
//                bOption = true;
//                break;
//            case 'a':
//                strArgument = cl.arg();
//            }
//        }
//    }
//    catch ( const SIMPLE_OPTIONS_EXCEPTION & e )
//    {
//        fprintf( stderr, "Invalid argument: %s\n", e.what() );
//        return 1;
//    }
//
// DISCLAIMER
// This code is released as freeware, you can do with it whatever you 
// like: use it, modify it, distribute it, sell it, delete it, or send 
// it to your mother-in-law.  I withhold no rights of any kind, and I 
// make no promises that this code will work correctly or at all.  
// Use it completely at your own risk.
//

#ifndef INCLUDED_CSimpleOptions
#define INCLUDED_CSimpleOptions

// redefine this exception type if you want to use something else
// it must take a string as a construction parameter
#ifndef SIMPLE_OPTIONS_EXCEPTION
#include <stdexcept>
#define SIMPLE_OPTIONS_EXCEPTION  std::invalid_argument
#endif

class CSimpleOptions
{
public:
    CSimpleOptions( int           a_argc, 
                    char *        a_argv[], 
                    const char *  a_pszOptions, 
                    bool          a_bCaseSensitive = true )
    {
        m_argc = a_argc;
        m_argv = a_argv;
        m_pszOptions = a_pszOptions;
        m_bCaseSensitive = a_bCaseSensitive;
        m_pszArg = 0;
        m_nCurr = 1;
    }

    // throws SIMPLE_OPTIONS_EXCEPTION
    char next()
    {
        // are we there yet?
        m_pszArg = 0;
        if ( m_nCurr >= m_argc )
        {
            return 0;
        }

        // ensure we have a switch in form "-c"
        if ( (m_argv[m_nCurr][0] != '-' && m_argv[m_nCurr][0] != '/')
             || m_argv[m_nCurr][1] == 0 || m_argv[m_nCurr][1] == ':' 
             || m_argv[m_nCurr][2] != 0 )
        {
            throw SIMPLE_OPTIONS_EXCEPTION( m_argv[m_nCurr] );
        }

        // find this argument
        m_pszArg = strchr( m_pszOptions, 
            m_bCaseSensitive ? m_argv[m_nCurr][1] : tolower( m_argv[m_nCurr][1] ) );
        if ( !m_pszArg )
        {
            throw SIMPLE_OPTIONS_EXCEPTION( m_argv[m_nCurr] );
        }

        // if we need an arg, ensure we have one
        if ( *(m_pszArg+1) == ':' ) 
        {
            if ( m_nCurr + 1 >= m_argc )
            {
                m_pszArg = 0;
                throw SIMPLE_OPTIONS_EXCEPTION( m_argv[m_nCurr] );
            }
            m_nCurr += 2;
        }
        else
        {
            ++m_nCurr;
        }

        return *m_pszArg;
    }

    // throw SIMPLE_OPTIONS_EXCEPTION
    const char * arg()
    {
        if ( *(m_pszArg+1) != ':' || m_nCurr <= 1 ) 
        {
            throw SIMPLE_OPTIONS_EXCEPTION( "" );
        }
        return m_argv[m_nCurr-1];
    }

    char option()
    {
        return m_pszArg ? *m_pszArg : (char) 0;
    }

private:
    int          m_argc;
    char **      m_argv;
    const char * m_pszOptions;
    bool         m_bCaseSensitive;
    char *       m_pszArg;
    int          m_nCurr;
};

#endif // INCLUDED_CSimpleOptions

