#include "reg6.h"

// TODO: Doxygen-compatible code comments.
// TODO: Use debug-flag

// This is the Observer of the Optimizer
class RegistrationCommandIterationUpdate : public itk::Command
{
public:
	typedef  RegistrationCommandIterationUpdate   Self;
	typedef  itk::Command             Superclass;
	typedef itk::SmartPointer<Self>  Pointer;
	itkNewMacro( Self );
protected:
	RegistrationCommandIterationUpdate() {};
public:
	typedef itk::VersorRigid3DTransformOptimizer OptimizerType;
	typedef const OptimizerType *OptimizerPointer;

	// Just call the const-execute when the object (Optimizer) is non-const.
	void Execute(itk::Object *caller, const itk::EventObject & event) {
		Execute( (const itk::Object *)caller, event);
	}

	// callback routine for the optimizer to invoke at end of iteration
	void Execute(const itk::Object * object, const itk::EventObject & event) {
		if( typeid( event ) != typeid( itk::IterationEvent ) ) {
			return;
		}
		OptimizerPointer optimizer = dynamic_cast< OptimizerPointer >( object );
		std::cout << optimizer->GetCurrentIteration() << " = ";
		std::cout << optimizer->GetValue() << " : ";
		std::cout << optimizer->GetCurrentPosition() << std::endl;
	}
};


int MattesMutualInformationRegistrationApplication::begin() {
	// Use types defined in header, get objects for registration
	TransformType::Pointer      transform     = TransformType::New();
	OptimizerType::Pointer      optimizer     = OptimizerType::New();
	InterpolatorType::Pointer   interpolator  = InterpolatorType::New();
	RegistrationType::Pointer   registration  = RegistrationType::New();
	MetricType::Pointer         metric        = MetricType::New();

	// Set all the registration objects
	registration->SetOptimizer(     optimizer     );
	registration->SetTransform(     transform     );
	registration->SetInterpolator(  interpolator  );
	registration->SetMetric(        metric        );

	// Define the input image readers
	FixedImageReaderType::Pointer  fixedImageReader  = FixedImageReaderType::New();
	MovingImageReaderType::Pointer movingImageReader = MovingImageReaderType::New();

	std::cout << "Using " << this->fixedFilename << " as fixed image file." << std::endl;
	fixedImageReader->SetFileName(  this->fixedFilename );
	std::cout << "Using " << this->movingFilename << " as moving image file." << std::endl;
	movingImageReader->SetFileName( this->movingFilename );

	// Have to pre-read this to enable finding scaling params below
	fixedImageReader->Update();
	movingImageReader->Update();
	fixedImageReader->GetImageIO()->ReadImageInformation();
	movingImageReader->GetImageIO()->ReadImageInformation();
	
	/*-------------------------------------------------------------------------------------------*/
	// Write out moving image to MHD/RAW image for verifying reading.
	if (this->writeoutinitialflag) {
		typedef itk::CastImageFilter< MovingImageType, OutputImageType > CastFirstMovingFilterType;
		typedef itk::ImageFileWriter< OutputImageType >  FirstMovingWriterType;

		FirstMovingWriterType::Pointer      firstMovingWriter =  FirstMovingWriterType::New();
		CastFirstMovingFilterType::Pointer  firstMovingCaster =  CastFirstMovingFilterType::New();

		firstMovingWriter->SetFileName( "testReadingMoving.mhd" );
		firstMovingCaster->SetInput( movingImageReader->GetOutput() );
		firstMovingWriter->SetInput( firstMovingCaster->GetOutput() );

		std::cout << "Writing out (just) read moving image...";
		std::cout.flush();

		firstMovingWriter->Update();

		std::cout << "done." << std::endl;
	}
	/*-------------------------------------------------------------------------------------------*/

	// Get image sizing parameters
	FixedImageType::SpacingType fixedImageSpacing = fixedImageReader->GetOutput()->GetSpacing();
	MovingImageType::SpacingType movingImageSpacing = movingImageReader->GetOutput()->GetSpacing();
	MovingImageType::RegionType movingImageRegion = movingImageReader->GetOutput()->GetLargestPossibleRegion();
	itk::Size<Dimension> movingImageSize = movingImageRegion.GetSize();
	FixedImageType::RegionType fixedImageRegion = fixedImageReader->GetOutput()->GetLargestPossibleRegion();
	itk::Size<Dimension> fixedImageSize = fixedImageRegion.GetSize();

	/*-------------------------------------------------------------------------------------------*/
	// Debug output to verify correct spacing is found.
	std::cout << "Spacing of Fixed Image: " << fixedImageSpacing[0] << ", " << fixedImageSpacing[1] << ", " << fixedImageSpacing[2] << std::endl;
	std::cout << "Spacing of Moving Image: " << movingImageSpacing[0] << ", " << movingImageSpacing[1] << ", " << movingImageSpacing[2] << std::endl;
	std::cout << "Dimensions of Fixed Image: " << fixedImageSize[0] << ", " << fixedImageSize[1] << ", " << fixedImageSize[2] << std::endl;
	std::cout << "Dimensions of Moving Image: " << movingImageSize[0] << ", " << movingImageSize[1] << ", " << movingImageSize[2] << std::endl;
	/*-------------------------------------------------------------------------------------------*/

	// Resample fixed image to 1/resampleDivider resolution all dimensions (smaller load)
	const int resampleDivider = 1;
	ResampleFixedFilterType::Pointer resampleFixedFilter = ResampleFixedFilterType::New();
	resampleFixedFilter->SetInput(fixedImageReader->GetOutput());
	std::cout << "Resampling fixed image from: " << std::endl;
	for (int i=0; i < Dimension; i++) {
		std::cout << "  [" << i << "]:" << fixedImageSize[i] << "px" << "(@" << fixedImageSpacing[i] << "mm/px) to ";
		fixedImageSize[i] = (long unsigned int) (fixedImageSize[i]/resampleDivider);
		fixedImageSpacing[i] = fixedImageSpacing[i]*resampleDivider;
		std::cout << fixedImageSize[i] << "px" << "(@" << fixedImageSpacing[i] << "mm/px)" << std::endl;
	}
	resampleFixedFilter->SetSize(fixedImageSize);
	resampleFixedFilter->SetOutputOrigin(fixedImageReader->GetOutput()->GetOrigin());
	resampleFixedFilter->SetOutputSpacing(fixedImageSpacing);

	//-------------------------------------------------------------------------------------------
	// Write out fixed image to MHD/RAW image for verifying first 1:resampleDivider -scaling.
	if (this->writeoutinitialflag) {
		typedef itk::CastImageFilter< FixedImageType, OutputImageType > InitZeroFixedCastFilterType;
		typedef itk::ImageFileWriter< OutputImageType >  InitZeroWriterType;

		InitZeroWriterType::Pointer            fixedZeroWriter =  InitZeroWriterType::New();
		InitZeroFixedCastFilterType::Pointer  fixedZeroCaster =  InitZeroFixedCastFilterType::New();

		fixedZeroWriter->SetFileName( "testFixedDownSampled.mhd" );
		fixedZeroCaster->SetInput( resampleFixedFilter->GetOutput() );
		fixedZeroWriter->SetInput( fixedZeroCaster->GetOutput() );

		std::cout << "Writing out fixed image after prelim. scaling...";
		std::cout.flush();

		fixedZeroWriter->Update();

		std::cout << "done." << std::endl;
	}
	//-------------------------------------------------------------------------------------------


	/* Resample the moving image so that with size (px) calculated here and spacing (mm) from
	 * the fixed image (i.e. they will have same spacing) the two images will have same size both physically
	 * and pixel-wise - thus, registration procedure will not have to worry about possible scaling.
	 *
	 * NB This only works when input images have set the scaling parameter (i.e. the imaging device is capable 
	 * of delivering physical spacing info, s.a. MRI, CT etc.)
	 */
	ResampleMovingFilterType::Pointer resampleMovingFilter = ResampleMovingFilterType::New();
	resampleMovingFilter->SetInput( movingImageReader->GetOutput() );

	std::cout << "Resampling moving image from: " << std::endl;
	for (int i=0; i < Dimension; i++) {
		std::cout << "  [" << i << "]:" << movingImageSize[i] << "px " << "(@" << movingImageSpacing[i] << "mm/px) to ";
		movingImageSize[i] = (long unsigned int) floor((movingImageSize[i]*(movingImageSpacing[i]/fixedImageSpacing[i])));
		std::cout << movingImageSize[i] << "px" << "(@" << fixedImageSpacing[i] << "mm/px)" << std::endl;
	}

	movingImageSpacing = fixedImageSpacing;
	// Using default transform (affine) with default transform params (0-matrix), giving only resampling (no "movement")
	resampleMovingFilter->SetSize(movingImageSize); // This is the calculated NEW size of the moving image!
	resampleMovingFilter->SetOutputOrigin(  movingImageReader->GetOutput()->GetOrigin() );
	resampleMovingFilter->SetOutputSpacing( movingImageSpacing);

	//-------------------------------------------------------------------------------------------
	// Write out each image to MHD/RAW image for verifying first scaling and resampling.
	if (this->writeoutinitialflag) {
		typedef itk::CastImageFilter< MovingImageType, OutputImageType > InitOneMovingCastFilterType;
		typedef itk::ImageFileWriter< OutputImageType >  InitOneWriterType;

		InitOneWriterType::Pointer      movingOneWriter =  InitOneWriterType::New();
		InitOneMovingCastFilterType::Pointer  movingOneCaster =  InitOneMovingCastFilterType::New();

		movingOneWriter->SetFileName( "testMovingDownsampled.mhd" );
		movingOneCaster->SetInput( resampleMovingFilter->GetOutput() );
		movingOneWriter->SetInput( movingOneCaster->GetOutput() );

		std::cout << "Writing out moving image after scaling (with 1:" << resampleDivider << " resampling)...";
		std::cout.flush();

		movingOneWriter->Update();

		std::cout << "done." << std::endl;
	}
	//-------------------------------------------------------------------------------------------

	//Casting to internal image type
	FixedCastFilterType::Pointer	fixedCastFilter = FixedCastFilterType::New();
	MovingCastFilterType::Pointer	movingCastFilter = MovingCastFilterType::New();
	fixedCastFilter->SetInput(resampleFixedFilter->GetOutput());
	movingCastFilter->SetInput(resampleMovingFilter->GetOutput());

	GaussianFilterType::Pointer fixedGaussianFilter = GaussianFilterType::New();
	GaussianFilterType::Pointer movingGaussianFilter = GaussianFilterType::New();
	GradMagFilterType::Pointer fixedGradMagFilter = GradMagFilterType::New();
	GradMagFilterType::Pointer movingGradMagFilter = GradMagFilterType::New();

	if(this->operation == 1) {
		//Bluring images prior to registration
		fixedGaussianFilter->SetVariance(this->sigmaFixed);
		movingGaussianFilter->SetVariance(this->sigmaMoving);
		fixedGaussianFilter->SetInput(fixedCastFilter->GetOutput());
		movingGaussianFilter->SetInput (movingCastFilter->GetOutput());
		std::cout << " We are using blurring!" << std::endl;
		std::cout << "Variance-fixed: " << this->sigmaFixed << ", variance-moving: " << this->sigmaMoving << std::endl;
	}
	else if(this->operation == 2) {
		// Find the Gradient Magnitude of each input image
		fixedGradMagFilter->SetSigma( this->sigmaFixed );
		movingGradMagFilter->SetSigma( this->sigmaMoving );	  
		fixedGradMagFilter->SetInput( fixedCastFilter->GetOutput() );
		movingGradMagFilter->SetInput( movingCastFilter->GetOutput() );
		std::cout << " We are using gradient!" << std::endl;
		std::cout << "Sigma-fixed: " << this->sigmaFixed << ", sigma-moving: " << this->sigmaMoving << std::endl;
	}
	 
	//-------------------------------------------------------------------------------------------
	// Write out images to verify gradient magnitude filtering. 
	if (this->writeoutinitialflag) {
		typedef itk::CastImageFilter< ImageType, OutputImageType > GradMagFixedCastFilterType;
		typedef itk::CastImageFilter< ImageType, OutputImageType > GradMagMovingCastFilterType;
		typedef itk::ImageFileWriter< OutputImageType >  GradMagWriterType;		
		GradMagWriterType::Pointer      movingGMWriter =  GradMagWriterType::New();
		GradMagMovingCastFilterType::Pointer  movingGMCaster =  GradMagMovingCastFilterType::New();
		GradMagWriterType::Pointer      fixedGMWriter =  GradMagWriterType::New();
		GradMagFixedCastFilterType::Pointer  fixedGMCaster =  GradMagFixedCastFilterType::New();		
		movingGMWriter->SetFileName( "testMovingGradMag.mhd" );
		fixedGMWriter->SetFileName(  "testFixedGradMag.mhd" );		  
		movingGMCaster->SetInput( movingGradMagFilter->GetOutput() );
		fixedGMCaster->SetInput( fixedGradMagFilter->GetOutput() );
		movingGMWriter->SetInput( movingGMCaster->GetOutput() );
		fixedGMWriter->SetInput( fixedGMCaster->GetOutput() );
		std::cout << "Writing out moving image after Gradient Magnitude filtering...";
		std::cout.flush();
		movingGMWriter->Update();
		fixedGMWriter->Update();
		std::cout << "done." << std::endl;
	}
	//-------------------------------------------------------------------------------------------
	  
	/*typedef itk::MinimumMaximumImageCalculator< ImageType > MinMaxCType;
	MinMaxCType::Pointer mmCalc = MinMaxCType::New();
	mmCalc->SetImage( fixedGradMagFilter->GetOutput() );
	std::cout << "Minimum/maximum in fixed (grad.mag.) image:" << std::endl;
	std::cout.flush();
	fixedGradMagFilter->Update();
	mmCalc->Compute();
	std::cout << " MIN:" << mmCalc->GetMinimum() << " - MAX:" << mmCalc->GetMaximum() << std::endl;
	   
	mmCalc->SetImage( movingGradMagFilter->GetOutput() );
	std::cout << "Minimum/maximum in moving image:" << std::endl;
	std::cout.flush();
	movingGradMagFilter->Update();
	mmCalc->Compute();
	std::cout << " MIN:" << mmCalc->GetMinimum() << " - MAX:" << mmCalc->GetMaximum() << std::endl;*/

	// Combine images and registration pipeline
	// Initialize the entire fixed image into the pipeline
	if(this->operation == 1) {
		registration->SetFixedImage(fixedGaussianFilter->GetOutput());
		registration->SetMovingImage(movingGaussianFilter->GetOutput());
		fixedGaussianFilter->Update(); // This is needed to make the BufferedRegion below valid.
		registration->SetFixedImageRegion( fixedGaussianFilter->GetOutput()->GetBufferedRegion() );
	}
	else if(this->operation == 2) {
		registration->SetFixedImage(fixedGradMagFilter->GetOutput());
		registration->SetMovingImage(movingGradMagFilter->GetOutput());
		fixedGradMagFilter->Update(); // This is needed to make the BufferedRegion below valid.
		registration->SetFixedImageRegion( fixedGradMagFilter->GetOutput()->GetBufferedRegion() );
	}
	else {
		registration->SetFixedImage(fixedCastFilter->GetOutput());
		registration->SetMovingImage(movingCastFilter->GetOutput());
		fixedCastFilter->Update(); // This is needed to make the BufferedRegion below valid.
		registration->SetFixedImageRegion( fixedCastFilter->GetOutput()->GetBufferedRegion() );
	}

	// Set the initial offset
	std::cout << "Setting initial offset (X,Y,Z): (" << this->initX << ", " << this->initY << ", " << this->initZ << ") -- and a zero versor (no rotations)...";
	std::cout.flush();

	ParametersType initialParameters(transform->GetNumberOfParameters());
	initialParameters[0] = this->initV1;  // Quaternion #1
	initialParameters[1] = this->initV2;  // Quaternion #2
	initialParameters[2] = this->initV3;  // Quaternion #3
	initialParameters[3] = this->initX;  // Initial offset in mm along X
	initialParameters[4] = this->initY;  // Initial offset in mm along Y
	initialParameters[5] = this->initZ;  // Initial offset in mm along Z
	registration->SetInitialTransformParameters( initialParameters );

	std::cout << "done." << std::endl;

	/*-------------------------------------------------------------------------------------------*/
	// Write out moving image to MHD/RAW image for verifying bluring and inital parameters.
	if (this->writeoutinitialflag) {
		TransformType::Pointer initialTransform = TransformType::New();

		initialTransform->SetParameters( initialParameters );

		InitialImageWriterResampleFilterType::Pointer initialResample = InitialImageWriterResampleFilterType::New();

		initialResample->SetTransform( initialTransform );
		initialResample->SetInput( movingGaussianFilter->GetOutput() );

		ImageType::Pointer initFixedImage = fixedGaussianFilter->GetOutput();

		initialResample->SetSize( movingGaussianFilter->GetOutput()->GetLargestPossibleRegion().GetSize() );
		initialResample->SetOutputOrigin(  initFixedImage->GetOrigin() );
		initialResample->SetOutputSpacing( initFixedImage->GetSpacing() );
		//initialResample->SetDefaultPixelValue( 0 );

		typedef itk::CastImageFilter< ImageType, OutputImageType > InitCastFilterType;
		typedef itk::ImageFileWriter< OutputImageType >  InitWriterType;

		InitWriterType::Pointer      movingWriter =  InitWriterType::New();
		InitCastFilterType::Pointer  movingCaster =  InitCastFilterType::New();

		movingWriter->SetFileName( "testMovingInitialTransformed.mhd" );
		movingCaster->SetInput( initialResample->GetOutput() );
		movingWriter->SetInput( movingCaster->GetOutput() );
		movingWriter->Update();
	}
	/*-------------------------------------------------------------------------------------------*/

	// Set the optimizer parameters
	double gradMagTol = optimizer->GetGradientMagnitudeTolerance();
	std::cout << "Setting optimizer parameters: " << this->numberOfIterations << " iterations , minStep: " << this->minStep << " maxStep: " << this->maxStep;
	std::cout << ", with gradient magnitude tolerance: " << gradMagTol << ", rotational scales=1.0, and minimize on...";
	optimizer->SetNumberOfIterations( this->numberOfIterations );
	optimizer->SetMaximumStepLength( this->maxStep );
	optimizer->SetMinimumStepLength( this->minStep );
	optimizer->MinimizeOn(); //Mattes should be minimized!

	OptimizerScalesType optimizerScales( transform->GetNumberOfParameters() );
	const double translationScale = (10.0 * sqrt((fixedImageSpacing[0]*fixedImageSize[0]*fixedImageSpacing[0]*fixedImageSize[0]) + (fixedImageSpacing[1]*fixedImageSize[1]*fixedImageSpacing[1]*fixedImageSize[1]) + (fixedImageSpacing[2]*fixedImageSize[2]*fixedImageSpacing[2]*fixedImageSize[2])));
	std::cout << " and translational scales= 1/" << translationScale << "..."; 
	optimizerScales[0] = 1.0; // scale for versor element 1
	optimizerScales[1] = 1.0; // scale for versor element 2
	optimizerScales[2] = 1.0; // scale for versor element 3
	optimizerScales[3] = 1.0 / translationScale; // scale for translation on X
	optimizerScales[4] = 1.0 / translationScale; // scale for translation on Y
	optimizerScales[5] = 1.0 / translationScale; // scale for translation on Z
	optimizer->SetScales( optimizerScales );
	std::cout << "done."  << std::endl;


	// Set the metric parameters
	std::cout << "Setting metric parameters: " << this->spatialSamples << " spatial samples, " << this->bins;
	std::cout << " bins. ";
	metric->SetNumberOfSpatialSamples(this->spatialSamples);
	metric->SetNumberOfHistogramBins(this->bins);
	metric->ReinitializeSeed();
	std::cout << "done." << std::endl;

    TransformType::InputPointType defCenter = transform->GetCenter();
    std::cout << "Center is set to: " << defCenter << "." << std::endl;

	// Register the observer with the optimizer.
	std::cout << "Setting the observer on the optimizer...";
	RegistrationCommandIterationUpdate::Pointer observer = RegistrationCommandIterationUpdate::New();
	optimizer->AddObserver( itk::IterationEvent(), observer );
	std::cout << "done." << std::endl;

	// Start Registration
	try
    {
		std::cout << "Starting registration!" << std::endl;
		std::cout.flush();
		registration->StartRegistration();
    }
	catch( itk::ExceptionObject & err )
    {
		std::cout << "ExceptionObject caught !" << std::endl;
		std::cout << err << std::endl;
		return -1;
    }
	// Registration done.

	// Get the parameters of the transformation at registration
	ParametersType finalParameters = registration->GetLastTransformParameters();
	const double Versor1 = finalParameters[0];
	const double Versor2 = finalParameters[1];
	const double Versor3 = finalParameters[2];
	const double TranslationAlongX = finalParameters[3];
	const double TranslationAlongY = finalParameters[4];
	const double TranslationAlongZ = finalParameters[5];

	numberOfIterations = optimizer->GetCurrentIteration();

	const double bestValue = optimizer->GetValue();
	itk::VersorRigid3DTransformOptimizer::StopConditionType stopCondition = optimizer->GetStopCondition();

	std::cout << "Registration done !" << std::endl;
	std::cout << "[stopped for reason (1=GradMagTolerance, 2=StepTooSmall, 3=ImageNotAvailable, 4=CostFuncErr, 5=MaxNumIter): " << stopCondition << "]" << std::endl;
	std::cout << "Number of iterations = " << numberOfIterations << std::endl;
	std::cout << "Translation along X  = " << TranslationAlongX << std::endl;
	std::cout << "Translation along Y  = " << TranslationAlongY << std::endl;
	std::cout << "Translation along Z  = " << TranslationAlongZ << std::endl;
	std::cout << "Versor (rotation)  = [" << Versor1 << "," << Versor2 << "," << Versor3 << "]" << std::endl;

	std::cout << "Optimal metric value = " << bestValue << std::endl;

	std::cout << "Comment = ITERS  XTRNS    YTRNS    ZTRNS     VERSOR            METRIC" << std::endl;
	std::cout << "Comment = " << numberOfIterations << " " << TranslationAlongX << " " << TranslationAlongY << " " << TranslationAlongZ << " (" << Versor1 << "," << Versor2 << "," << Versor3 << ") " << bestValue << std::endl;

	// Prepare the resampling filter in order to map the moving image.
	// TODO: Use the original moving image, back-scale transformation
	// parameters acc to initial 1:4 downsampling (updcale 4:1, considering...?)



	ResampleFinalFilterType::Pointer resample = ResampleFinalFilterType::New();
	resample->SetTransform(registration->GetOutput()->Get());
	resample->SetInput(movingImageReader->GetOutput());

	FixedImageType::Pointer fixedImage = fixedImageReader->GetOutput();

	resample->SetSize( fixedImage->GetLargestPossibleRegion().GetSize() );
	resample->SetOutputOrigin(  fixedImage->GetOrigin() );
	resample->SetOutputSpacing( fixedImage->GetSpacing() );
	resample->SetDefaultPixelValue( 0 );

	//CHECK OUT THIS WITH ERIK!!!!!!!!
	//TransformType::Pointer finalTransform = TransformType::New();
	//TransformType::Pointer finalInverseTransform = TransformType::New();
	//finalTransform->SetParameters( finalParameters );
	//if (!finalTransform->GetInverse( finalInverseTransform )) {
	//	std::cout << "ERROR: Could not get inverse transform!" << std::endl << "Quitting!" << std::endl;
	//  	return(8);
	//}
	//else {
	//  	std::cout << "Got reverse transform..." << std::endl;
	//}
	//resample->SetTransform( finalInverseTransform );
	//resample->SetInput( fixedImage );
	//FixedImageType::Pointer fixedImage = resampleFixedFilter->GetOutput();


	// Prepare a writer and caster filters to send the resampled moving image to
	// a file
	//

	WriterType::Pointer		writer = WriterType::New();
	CastFilterType::Pointer caster = CastFilterType::New();

	writer->SetFileName( this->outputFilename );

	caster->SetInput( resample->GetOutput() );
	writer->SetInput( caster->GetOutput()   );
	writer->Update();

	return 0;
}


void MattesMutualInformationRegistrationApplication::setMovingFilename(char *filename) {
	this->movingFilename = filename;
}

void MattesMutualInformationRegistrationApplication::setFixedFilename(char *filename) {
	this->fixedFilename = filename;
}

void MattesMutualInformationRegistrationApplication::setOutputFilename(char *filename) {
	this->outputFilename = filename;
}

char *MattesMutualInformationRegistrationApplication::getMovingFilename() {
	return this->movingFilename;
}

char *MattesMutualInformationRegistrationApplication::getFixedFilename() {
	return this->fixedFilename;
}

char *MattesMutualInformationRegistrationApplication::getOutputFilename() {
	return this->outputFilename;
}

void MattesMutualInformationRegistrationApplication::setSpatialSamples(unsigned int samples) {
	this->spatialSamples = samples;
}

void MattesMutualInformationRegistrationApplication::setNumberOfBins(unsigned int bins) {
	this->bins = bins;
}

void MattesMutualInformationRegistrationApplication::setNumberOfIterations(unsigned int iterations) {
	this->numberOfIterations = iterations;
}

void MattesMutualInformationRegistrationApplication::setMaxStep(double maxStep) {
	this->maxStep = maxStep;
}

void MattesMutualInformationRegistrationApplication::setMinStep(double minStep) {
	this->minStep = minStep;
}

void MattesMutualInformationRegistrationApplication::setSigmaFixed(double sigmaFixed) {
	this->sigmaFixed = sigmaFixed;
}

void MattesMutualInformationRegistrationApplication::setSigmaMoving(double sigmaMoving) {
	this->sigmaMoving = sigmaMoving;
}

void MattesMutualInformationRegistrationApplication::setInitX(double initX) {
	this->initX = initX;
}

void MattesMutualInformationRegistrationApplication::setInitY(double initY) {
	this->initY = initY;
}

void MattesMutualInformationRegistrationApplication::setInitZ(double initZ) {
	this->initZ = initZ;
}

void MattesMutualInformationRegistrationApplication::setOperation(unsigned int operation) {
	this->operation = operation;
}

void MattesMutualInformationRegistrationApplication::setDebugFlag(unsigned int dflag) {
	this->debugflag = dflag;
}

void MattesMutualInformationRegistrationApplication::setWriteOutInitialFlag(unsigned int wflag) {
	this->writeoutinitialflag = wflag;
}

int main(int argc, char *argv[]) {
	MattesMutualInformationRegistrationApplication::Pointer regApp = new MattesMutualInformationRegistrationApplication();

    try {
		CSimpleOptions cl( argc, argv, "f:m:o:i:X:n:s:b:v:V:x:y:z:a:dwh" );
		while (cl.next()) {
			switch (cl.option()) {
    			case 'f':	//Fixed image
  					regApp->setFixedFilename((char *)cl.arg());
  					std::cout << " fixedFilename set!" << std::endl;
	  				break;
				case 'm':		//Moving image
					regApp->setMovingFilename((char *)cl.arg());
	  				std::cout << " movingFilename set!" << std::endl;
  					break;
  				case 'o':		//Output image
  					regApp->setOutputFilename((char *)cl.arg());
	  				std::cout << " outputFilename set!" << std::endl;
  					break;
	  			case 'i':		//Iterations
  					regApp->setNumberOfIterations((unsigned int) floor(atof(cl.arg())));
	  				std::cout << " numberOfIterations set!" << std::endl;
  					break;
				case 'X':		//Max-step
  					regApp->setMaxStep((double) atof(cl.arg()));
	  				std::cout << " maxStep set!" << std::endl;
  					break;
	  			case 'n':		//Min-step
  					regApp->setMinStep((double) atof(cl.arg()));
	  				std::cout << " minStep set!" << std::endl;
  					break;
				case 's':		//Samples
  					regApp->setSpatialSamples((unsigned int) floor(atof(cl.arg())));
	  				std::cout << " spatialSamples set!" << std::endl;
  					break;
				case 'b':		//Bins
  					regApp->setNumberOfBins((unsigned int) floor(atof(cl.arg())));
	  				std::cout << " histogramBins set!" << std::endl;
  					break;
				case 'v':		//GradMag fixed
					regApp->setSigmaFixed((double) atof(cl.arg()));
					std::cout << " sigmaFixed set!" << std::endl;
					break;
				case 'V':		//GradMag fixed
					regApp->setSigmaMoving((double) atof(cl.arg()));
					std::cout << " sigmaMoving set!" << std::endl;
					break;
	  			case 'x':		//X-init
  					regApp->setInitX((double) atof(cl.arg()));
	  				std::cout << " initX set!" << std::endl;
  					break;
	  			case 'y':		//Y-init
  					regApp->setInitY((double) atof(cl.arg()));
	  				std::cout << " initY set!" << std::endl;
  					break;
	  			case 'z':		//Z-init
  					regApp->setInitZ((double) atof(cl.arg()));
	  				std::cout << " initZ set!" << std::endl;
  					break;
				case 'a':		//Operation
  					regApp->setOperation((unsigned int) floor(atof(cl.arg())));
	  				std::cout << " Operation set!" << std::endl;
  					break;
	  			case 'd':		//Debug
  					regApp->setDebugFlag(1);
	  				std::cout << " debug set!" << std::endl;
  					break;
	  			case 'w':		//Write out initial
  					regApp->setWriteOutInitialFlag(1);
  					std::cout << " writeoutinitial set!" << std::endl;
  					break;
				case 'h':
					regApp->usage();
					break;
		  		default:
  					regApp->usage();
			}
		}
	}
    catch (const SIMPLE_OPTIONS_EXCEPTION & e) {
		fprintf( stderr, "Invalid argument: %s\n", e.what() );
		regApp->usage();
        return 1;
    }

	if (strcmp(regApp->getMovingFilename(), "") == 0 || strcmp(regApp->getFixedFilename(), "") == 0 || strcmp(regApp->getOutputFilename(), "") == 0) {
  		regApp->usage();
		return 1;
	}

	return regApp->begin();
}
